﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Text;

namespace KYSMSDemo {
    class Program {

        public static string DEF_CHATSET = "UTF-8";
        public static int DEF_CONN_TIMEOUT = 30000;
        public static string USERAGENT = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
        public static string URL = "URL,联系商务获取";
        public static string USERID = "用户ID,联系商务获取";
        public static string APIKEY = "加密key,联系商务获取";

        static void Main(string[] args) {

            // 1、发送短信
            String phone = "填写手机号码";  //手机号码，多手机号用逗号分隔如 13300000000,13200000000
            String content = "【五星集团】您的验证码为：731848，有效期为30分钟，非本人发送敬请忽略"; //短信内容(内容前面必须要带上签名如:【五星集团】)
            MT(phone, content);


            // 2、查询剩余条数
             Balance();

            // 3、短信状态报告查询接口
             MTQuery();

            //4、短信上行查询
            MoQuery();

            Console.ReadLine();

        }

        /// <summary>
        /// 发送短信
        /// </summary>
        /// <param name="phones">不能为空，手机号码，多手机号用逗号分隔如 13300000000,13200000000</param>
        /// <param name="content">不能为空，短信内容(内容前面必须要带上签名如:【宽信科技】)</param>
        public static void MT(string phones, string content) {
            String url = URL + "/api/sms/send";
            String ts = GetTimestamp();
            String md5 = Md5Hash(USERID + ts + APIKEY);
            Dictionary<string, object> dic = new Dictionary<string, object>
            {
                { "userid",USERID},
                { "ts",ts},
                { "sign",md5.ToLower()},
                { "mobile",phones},
                { "msgcontent",content}
            };
            try {
                String result = Post(url, "", dic, DEF_CONN_TIMEOUT);
                Console.Write(result);
            } catch (Exception e) {
                Console.WriteLine(e.Message);
            }
        }

        /// <summary>
        /// 短信状态报告查询
        /// </summary>
        public static void MTQuery() {
            String url = URL + "/api/v2/sms/query";
            String ts = GetTimestamp();
            String md5 = Md5Hash(USERID + ts + APIKEY);
            Dictionary<string, object> dic = new Dictionary<string, object>
            {
                { "userid",USERID},
                { "ts",ts},
                { "sign",md5.ToLower()},
            };
            try {
                String result = Post(url, null, dic, DEF_CONN_TIMEOUT);
                Console.Write(result);
            } catch (Exception e) {
                Console.WriteLine(e.Message);
            }
        }

        /// <summary>
        /// 剩余条数查询
        /// </summary>
        public static void Balance() {
            String url = URL + "/api/sms/balance";
            String ts = GetTimestamp();
            String md5 = Md5Hash(USERID + ts + APIKEY);
            Dictionary<string, object> dic = new Dictionary<string, object>
            {
                { "userid",USERID},
                { "ts",ts},
                { "sign",md5.ToLower()},
            };
            try {
                String result = Post(url, null, dic, DEF_CONN_TIMEOUT);
                Console.Write(result);
            } catch (Exception e) {
                Console.WriteLine(e.Message);
            }
        }

        /// <summary>
        /// 短信上行查询
        /// </summary>
        public static void MoQuery() {
            String url = URL + "/api/v2/sms/moquery";
            String ts = GetTimestamp();
            String md5 = Md5Hash(USERID + ts + APIKEY);
            Dictionary<string, object> dic = new Dictionary<string, object>
            {
                { "userid",USERID},
                { "ts",ts},
                { "sign",md5.ToLower()},
            };
            try {
                String result = Get(url, null, dic, DEF_CONN_TIMEOUT);
                Console.Write(result);
            } catch (Exception e) {
                Console.WriteLine(e.Message);
            }
        }

        /// <summary>
        /// 获取当前时间的时间戳
        /// </summary>
        /// <returns></returns>
        private static string GetTimestamp() {
            TimeSpan ts = DateTime.Now.ToUniversalTime() - new DateTime(1970, 1, 1);
            return ((long)ts.TotalMilliseconds).ToString(); //按毫秒计算的时间戳
        }

        /// <summary>
        /// md5加密
        /// </summary>
        /// <param name="str">待加密字串</param>
        /// <returns></returns>
        private static string Md5Hash(string str) {
            byte[] bytes;
            MD5 md5 = MD5.Create();
            bytes = md5.ComputeHash(Encoding.UTF8.GetBytes(str));
            StringBuilder result = new StringBuilder();
            for (int i = 0; i < bytes.Length; i++) {
                result.Append(bytes[i].ToString("x2"));
            }
            return result.ToString();
        }

        /// <summary>
        /// 构建请求参数字符串
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        private static string BuildParams(Dictionary<string, object> data) {
            StringBuilder sb = new StringBuilder();
            foreach (KeyValuePair<string, object> kv in data) {
                sb.Append($"{kv.Key}={kv.Value}&");
            }
            return sb.ToString().TrimEnd('&');//去掉最后1个多余的“&”
        }

        /// <summary>
        /// http post请求
        /// </summary>
        /// <param name="requestUrl">请求URL</param>
        /// <param name="requestContentType">Content-Type</param>
        /// <param name="requestData">请求参数</param>
        /// <param name="requestTimeoutSeconds">请求超时秒数</param>
        /// <returns></returns>
        private static string Post(string requestUrl, string requestContentType = null, Dictionary<string, object> requestData = null, int? requestTimeoutSeconds = null) {
            string result = null;//请求结果
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUrl);
            request.Method = "POST";
            request.Timeout = (requestTimeoutSeconds ?? 10) * 1000;
            request.UserAgent = USERAGENT;
            if (!string.IsNullOrEmpty(requestContentType)) {
                request.ContentType = requestContentType;
            } else {
                request.ContentType = "application/x-www-form-urlencoded";
            }
            try {
                if (requestData != null && requestData.Count > 0) {
                    string requestParams = BuildParams(requestData);
                    byte[] data = Encoding.UTF8.GetBytes(requestParams);
                    request.ContentLength = data.Length;
                    using (Stream reqStream = request.GetRequestStream()) {
                        reqStream.Write(data, 0, data.Length);
                        reqStream.Close();
                    }
                }
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream stream = response.GetResponseStream();
                using (StreamReader reader = new StreamReader(stream, Encoding.UTF8)) {
                    result = reader.ReadToEnd();
                }
            } catch (Exception ex) {
                //throw ex;
                Console.WriteLine(ex.Message);
            }
            return result;
        }


        /// <summary>
        /// http get 请求
        /// </summary>
        /// <param name="requestUrl">请求URL</param>
        /// <param name="requestContentType">Content-Type</param>
        /// <param name="requestData">请求参数</param>
        /// <param name="requestTimeoutSeconds">请求超时秒数</param>
        /// <returns></returns>
        private static string Get(string requestUrl, string requestContentType = null, Dictionary<string, object> requestData = null, int? requestTimeoutSeconds = null) {
            string result = null;//请求结果
            if (requestData != null && requestData.Count > 0) {
                string requestParams = BuildParams(requestData);
                requestUrl += "?" + requestParams;
            }
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUrl);
            request.Method = "GET";
            request.Timeout = (requestTimeoutSeconds ?? 10) * 1000;
            request.UserAgent = USERAGENT;
            if (!string.IsNullOrEmpty(requestContentType)) {
                request.ContentType = requestContentType;
            } else {
                request.ContentType = "application/x-www-form-urlencoded";
            }
            try {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream stream = response.GetResponseStream();
                using (StreamReader reader = new StreamReader(stream, Encoding.UTF8)) {
                    result = reader.ReadToEnd();
                }
            } catch (Exception ex) {
                //throw ex;
                Console.WriteLine(ex.Message);
            }
            return result;
        }
    }
}
